# -*- coding: utf-8 -*- 
# @Time    : 04/05/2024 
# @Author  : nzooherd 
# @File    : watchdog_app.py 
# @Software: Visual Studio Code

import asyncio
from logging import getLogger
import os
from threading import Thread
import time

import psutil
from sqlalchemy import true
from bootstrap import BaseApp

from watchdog.observers import Observer
from watchdog.events import FileSystemEvent, FileSystemEventHandler

from config import Config
from service.resource_service import ResourceService

logger = getLogger()

class ResourceEventHandler(FileSystemEventHandler):

    def __init__(self) -> None:
        super().__init__()
        self.resource_service = ResourceService()
        try:
            self.loop = asyncio.get_event_loop()
        except RuntimeError:
            self.loop = asyncio.new_event_loop()
            asyncio.set_event_loop(self.loop)
        
        Thread(target=self.loop.run_until_complete, args=(self.start(), ), daemon=False).start()

        logger.info("ResourceEventHandler started")
        
    async def start(self):
        while true:
            await asyncio.sleep(60)
        
    def on_created(self, event: FileSystemEvent) -> None:
        base_path = os.path.dirname(event.src_path)
        logger.info(f"File created: {event.src_path}")
        for ignore in ["clip", "temp", "preview"]:
            if ignore in os.path.basename(event.src_path):
                return
            
        if base_path == os.path.join(Config.RESOURCE_ROOT_PATH, "upload"):
            self.loop.create_task(self.resource_service.update_online_url(event.src_path))
        if base_path == os.path.join(Config.RESOURCE_ROOT_PATH, "preview"):
            self.loop.create_task(self.resource_service.update_preview_url(event.src_path))
        if base_path == os.path.join(Config.RESOURCE_ROOT_PATH, "metadata"):
            self.loop.create_task(self.resource_service.update_metadata(event.src_path))
    
class WatchdogApp(BaseApp):

    def __init__(self):
        super().__init__()
        event_handler = ResourceEventHandler()
        self.name = "123"
        self.observer = Observer()
        self.observer.schedule(event_handler, Config.RESOURCE_ROOT_PATH, recursive=true)
        self.observer.start()
        print(id(self))
    
    def start(self):
        try:
            while True:
                time.sleep(1)
        finally:
            self.observer.stop()
            self.observer.join()
    

def get_all_children(pid):
    try:
        parent = psutil.Process(pid)
    except psutil.NoSuchProcess:
        return []
    
    children = []
    for child in parent.children(recursive=True):
        try:
            children.append(child)
        except psutil.NoSuchProcess:
            pass
    return children

if __name__ == '__main__':
    WatchdogApp().start()